use adventofcode2023::utils::get_puzzle;

// Convert spelled numbers to digits.
// The order of the replacements is optimized to minimize
// the overlapping trailing characters to keep at each step.
fn convert_spelled_numbers(line: &str) -> String {
    line.replace("four", "4")
        .replace("six", "6")
        .replace("eight", "e8t")
        .replace("three", "3")
        .replace("five", "5")
        .replace("one", "o1")
        .replace("two", "2")
        .replace("seven", "7n")
        .replace("nine", "9")
}

fn compute_calibration_values_sum(puzzle: &[u8]) -> u32 {
    puzzle
        .split(|&c| c == b'\n')
        .map(|line| {
            ((line.iter().find(|c| c.is_ascii_digit()).unwrap() - b'0') * 10
                + (line.iter().rev().find(|c| c.is_ascii_digit()).unwrap() - b'0'))
                as u32
        })
        .sum()
}

fn main() {
    let puzzle = get_puzzle();
    let start_time = std::time::Instant::now();
    println!("--- Day 1: Trebuchet?! ---");
    println!("What is the sum of all of the calibration values?");
    println!("{}", compute_calibration_values_sum(puzzle.as_bytes()));
    println!("--- Part Two ---");
    println!("What is the sum of all of the calibration values?");
    let converted_puzzle = convert_spelled_numbers(&puzzle);
    println!(
        "{}",
        compute_calibration_values_sum(converted_puzzle.as_bytes())
    );
    let elapsed_time = start_time.elapsed();
    println!("compute time: {:?}", elapsed_time);
}
