use adventofcode2023::utils::get_puzzle_bytes;

fn compute_points(winning_numbers: &[u8], my_numbers: &[u8]) -> u32 {
    let win_count = my_numbers
        .chunks_exact(3)
        .map(|n| &n[1..])
        .filter(|n| {
            winning_numbers
                .chunks_exact(3)
                .map(|n| &n[1..])
                .any(|c| &c == n)
        })
        .count() as u32;
    2u32.pow(win_count) >> 1
}

fn compute_total_points(puzzle: &[u8]) -> u32 {
    let col = puzzle.iter().position(|&b| b == b':').unwrap();
    let sep = puzzle.iter().position(|&b| b == b'|').unwrap();
    puzzle
        .split(|&c| c == b'\n')
        .map(|line| (&line[col + 1..sep], &line[sep + 1..]))
        .map(|(winning_numbers_bytes, my_numbers_bytes)| {
            compute_points(winning_numbers_bytes, my_numbers_bytes)
        })
        .sum()
}

struct Scratchcard {
    score: u32,
    copies: u32,
}

fn parse_scratchcards(puzzle: &[u8]) -> Vec<Scratchcard> {
    let col = puzzle.iter().position(|&b| b == b':').unwrap();
    let sep = puzzle.iter().position(|&b| b == b'|').unwrap();
    puzzle
        .split(|&c| c == b'\n')
        .map(|line| (&line[col + 1..sep], &line[sep + 1..]))
        .map(|(winning_numbers, my_numbers)| Scratchcard {
            score: my_numbers
                .chunks_exact(3)
                .map(|n| &n[1..])
                .filter(|n| {
                    winning_numbers
                        .chunks_exact(3)
                        .map(|n| &n[1..])
                        .any(|c| &c == n)
                })
                .count() as u32,
            copies: 1,
        })
        .collect::<Vec<Scratchcard>>()
}

fn compute_total_scratchcards(puzzle: &[u8]) -> u32 {
    let mut scratchcards = parse_scratchcards(puzzle);
    for current_i in 0..scratchcards.len() {
        let to_copy_end = current_i + 1 + scratchcards[current_i].score as usize;
        let current_copies = scratchcards[current_i].copies;
        for next_scratchcard in scratchcards
            .iter_mut()
            .take(to_copy_end)
            .skip(current_i + 1)
        {
            next_scratchcard.copies += current_copies;
        }
    }
    scratchcards.iter().map(|card| card.copies).sum()
}

fn main() {
    let puzzle = get_puzzle_bytes();
    let start_time = std::time::Instant::now();
    println!("--- Day 4: Scratchcards ---");
    println!("How many points are they worth in total?");
    println!("{}", compute_total_points(&puzzle));
    println!("--- Part Two ---");
    println!("how many total scratchcards do you end up with?");
    println!("{}", compute_total_scratchcards(&puzzle));
    let elapsed_time = start_time.elapsed();
    println!("compute time: {:?}", elapsed_time);
}
